/***************************************************************************
                          globject.cpp  -  description
                             -------------------
    begin                : Fre Feb 7 2003
    copyright            : (C) 2003 by Roman Roelofsen
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "globject.h"
#include <qcolor.h>

#include "glbox.h"
#include "glcube.h"
#include "glsnubcube.h"
#include "glcubecross.h"
#include "glcross.h"
#include "glocta.h"

// Create a GLObject widget
GLObject::GLObject( QWidget* parent, const char* name ) : QGLWidget( parent, name )
{
	xRot = yRot = zRot = 0.0;	// default object rotation
	scale = 1.0;			// default object scale
	object = 0;

	fg_color = QColor(28, 156, 150);
	bg_color = QColor(0, 0, 0);
}

// Release allocated resources
GLObject::~GLObject()
{
	makeCurrent();
    	glDeleteLists( object, 1 );
}

// Paint the box. The actual openGL commands for drawing the box are performed here.
void GLObject::paintGL()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -10.0 );
	glScalef( scale, scale, scale );
	
	// rotate animation
	glRotatef( xRot, 1.0, 0.0, 0.0 );
	glRotatef( yRot, 0.0, 1.0, 0.0 );
	glRotatef( zRot, 0.0, 0.0, 1.0 );

	// draw object
	glCallList( object );

	// lights
	GLfloat pos0[4] = { 7.0f, 0.0f, 4.0f, 1.0 };
 	glLightfv( GL_LIGHT0, GL_POSITION, pos0 );
        glEnable( GL_LIGHT0 );
         
	GLfloat pos1[4] = { 5.0f, 5.0f, 5.0f, 1.0f };
	GLfloat acolor[4] = { 0.7f, 0.7f, 0.7f, 0.5f };
 	glLightfv( GL_LIGHT1, GL_POSITION, pos1 );
  	glLightfv( GL_LIGHT1, GL_AMBIENT, acolor) ;
       	glEnable( GL_LIGHT1 );         
}

// Set up the OpenGL rendering state, and define display list
void GLObject::initializeGL()
{
	// global settings
 	glEnable( GL_CULL_FACE );
	glEnable( GL_LIGHTING );
       	glEnable( GL_DEPTH_TEST );

	// BG-color
	qglClearColor( bg_color );
        
	// FG-color
	GLfloat glmc[4] = {	(GLfloat) (fg_color.red() / 255.0) * 0.7,
				(GLfloat) (fg_color.green() / 255.0) * 0.7,
				(GLfloat) (fg_color.blue() / 255.0) * 0.7, 1.0f };
	glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, glmc );
	
	// create object
	if ( o_name == "Box" ) 		object = GLBox::makeObject();
	if ( o_name == "Cross" )	object = GLCross::makeObject();
	if ( o_name == "Cube" ) 	object = GLCube::makeObject();
	if ( o_name == "CubeCross" )	object = GLCubeCross::makeObject();	 
	if ( o_name == "Octa" ) 	object = GLOcta::makeObject();
	if ( o_name == "SnubCube" ) 	object = GLSnubCube::makeObject();

	glEnable( GL_NORMALIZE );
}

// Set up the OpenGL view port, matrix mode, etc.
void GLObject::resizeGL( int w, int h )
{
	glViewport( 0, 0, (GLint)w, (GLint)h );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	glFrustum( -1.0, 1.0, -1.0, 1.0, 5.0, 100.0 );
	glMatrixMode( GL_MODELVIEW );
}

void GLObject::setRotation(int x, int y, int z)
{
	xRot = (GLfloat)(x % 360);
	yRot = (GLfloat)(y % 360);
	zRot = (GLfloat)(z % 360);	
	updateGL();
}

void GLObject::setColors(QColor new_fg_color, QColor new_bg_color)
{
	if (new_fg_color != fg_color || new_bg_color != bg_color) {
		fg_color = new_fg_color;
		bg_color = new_bg_color;
		initializeGL();
	}
}

void GLObject::setObject(QString new_o_name)
{
	if (new_o_name != o_name) {
		o_name = new_o_name;
		initializeGL();
	}
}


