/***********************************************************************************
		        	kcube.cpp
                             -------------------
    begin                : Mon Jan 27 19:29:59 CET 2003
    copyright           : (C) 2003 by Roman Roelofsen
 ***********************************************************************************

 ***********************************************************************************
 *										
 *   This program is free software; you can redistribute it and/or modify  	
 *   it under the terms of the GNU General Public License as published by  	
 *   the Free Software Foundation; either version 2 of the License, or     	
 *   (at your option) any later version.                                   		
 *
 ***********************************************************************************
 */

#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kcolorbutton.h>
#include <krun.h>

#include <qframe.h>
#include <qstring.h>
#include <qcolor.h>
#include <qfont.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qslider.h>

#include <stdio.h>

#include "kcube.h"

extern "C"
{
	KPanelApplet* init( QWidget *parent, const QString configFile)
	{
		KGlobal::locale()->insertCatalogue("kcube");
		return new Kcube(configFile, KPanelApplet::Normal, KPanelApplet::About | KPanelApplet::Preferences, parent, "kcube");
	}	
}

Kcube::Kcube(const QString& configFile, Type type, int actions, QWidget *parent, const char *name)  : KPanelApplet(configFile, type, actions, parent, name)
{
	// OpenGL-Object
	globject = new GLObject(this, "globject");

	// Label
	label_load = new QLabel( this, "label_load" );
	label_load->setAlignment( int( QLabel::AlignCenter ) );
	label_load->setFixedHeight(10);
	label_load->setFont( QFont("Helvetica", 8, QFont::Light) );
	label_load->setText("0%");

	// Layout
	vlayout = new QVBoxLayout(this, 0, 0, 0);
	vlayout->addWidget( globject );
	vlayout->addWidget( label_load );

	// timer
	cpu_timer = new QTimer();
	cpu_timer->start(500, false);
	connect (cpu_timer, SIGNAL ( timeout() ), this, SLOT ( cpu_timer_event() ) );
	draw_timer = new QTimer();
	draw_timer->start(50, false);
	connect (draw_timer, SIGNAL( timeout() ), this, SLOT ( draw_timer_event() ) );

	// calc_cpu_total() init values
	previous_total = 0;
	previous_used = 0;

	// OpenGL init values
	rot_x = rot_y = rot_z = 0;

	// config
	config = new KSimpleConfig ("kcuberc");
	prefform = new PrefForm(this);
	prefform->setFixedSize( prefform->width(), prefform->height() );
	loadConfig();
}

Kcube::~Kcube()
{
}

// show about-dialog
void Kcube::about()
{
	KMessageBox::information(0, "Kcube 0.61 \nwmcube for KDE / Kicker\n(C) 2003, Roman Roelofsen\nmailto: develop@tuxed.de");
}

// show configuration-dialog
void Kcube::preferences()
{
	loadConfig();
	if (prefform->exec () == QDialog::Accepted) {
		saveConfig();
		loadConfig();
	}
}

int Kcube::widthForHeight( int height ) const
{
	return height;
}

int Kcube::heightForWidth( int width ) const
{
	return width;
}


// calculate cpuload
int Kcube::calc_cpu_total()
{
	int 	use_nice = 0, t=0;
	int 	total, used;
	char 	cpuid[6];
	int 	cpu,nice,system,idle;

	FILE *fp;
	fp = fopen("/proc/stat","rt");
	fscanf(fp,"%s %d %d %d %d",cpuid,&cpu,&nice,&system,&idle);
	fclose(fp);

	used 	= cpu + system + nice;
	total 	= used + idle + (1-use_nice)*nice;

	t = (int)(100 * (double)(used - previous_used) / (double)(total - previous_total));
	previous_total = total;
	previous_used = used;
	
	if (t < 0) t = 0;
	if (t > 100) t = 100;
	return t;
}

// calculate cpuload, update label_cpu
void Kcube::cpu_timer_event()
{
	load = calc_cpu_total();

	if ( load > prefform->high_color_cpu->text().toInt() ) setColors("high");
	else setColors("low");
		
	draw_timer->changeInterval( 51 - load/2 );
	label_load->setText( QString("%1%").arg(load) );
}

// draw the 3D-modell, rotate
void Kcube::draw_timer_event()
{
	if ( prefform->x_rot_speed->value() == 0 )	rot_x = prefform->s_x_rot->text().toInt();
	else						rot_x = (rot_x  + prefform->x_rot_speed->value() ) % 360;
	if ( prefform->y_rot_speed->value() == 0 ) 	rot_y = prefform->s_y_rot->text().toInt();
	else						rot_y = (rot_y + prefform->y_rot_speed->value() ) % 360;
	if ( prefform->z_rot_speed->value() == 0 ) 	rot_z = prefform->s_z_rot->text().toInt();
	else						rot_z = (rot_z + prefform->z_rot_speed->value() ) % 360;
	
	globject->setRotation(rot_x, rot_y, rot_z);
}

// load configuration
void Kcube::loadConfig()
{
	config->sync ();
	config->setGroup("Kcube Applet");

	// Mouse button Click
	prefform->lmb_click->setText( config->readEntry("lmb_click", "ksysguard") );
	prefform->rmb_click->setText( config->readEntry("rmb_click", "") );
	
	// 3D object colors
	prefform->obj_fg_color_l->setColor( config->readColorEntry("obj_fg_color_l", new QColor(44, 144, 215) ) );
	prefform->obj_fg_color_h->setColor( config->readColorEntry("obj_fg_color_h", new QColor(255, 0, 0) ) );
	prefform->obj_bg_color_l->setColor( config->readColorEntry("obj_bg_color_l", new QColor(255, 255, 255) ) );
	prefform->obj_bg_color_h->setColor( config->readColorEntry("obj_bg_color_h", new QColor(255, 255, 255) ) );

	// cpu colors
	prefform->cpu_fg_color_l->setColor( config->readColorEntry("cpu_fg_color_l", new QColor(44, 144, 215) ) );
	prefform->cpu_fg_color_h->setColor( config->readColorEntry("cpu_fg_color_h", new QColor(255, 0, 0) ) );
	prefform->cpu_bg_color_l->setColor( config->readColorEntry("cpu_bg_color_l", new QColor(255, 255, 255) ) );
	prefform->cpu_bg_color_h->setColor( config->readColorEntry("cpu_bg_color_h", new QColor(255, 255, 255) ) );
						
	setColors("low");
	
	// high color cpu usage
	prefform->high_color_cpu->setValue ( config->readNumEntry("high_color_cpu", 80) );
																																													
	// 3D Object
	prefform->s_3dobject->setSelected( prefform->s_3dobject->findItem( config->readEntry("3d_object", "Cube") ), true );
	globject->setObject( prefform->s_3dobject->currentText() );

	// Refreshrate
	prefform->r_rate->setValue( config->readNumEntry("r_rate", 250));
	cpu_timer->changeInterval( prefform->r_rate->text().toInt() );

	// rotation
	prefform->s_x_rot->setValue ( 		config->readNumEntry("s_x_rot", 0) );
	prefform->s_y_rot->setValue ( 		config->readNumEntry("s_y_rot", 0) );
	prefform->s_z_rot->setValue ( 		config->readNumEntry("s_z_rot", 0) );
	prefform->x_rot_speed->setValue (	config->readNumEntry("x_rot_speed", 3) );
	prefform->y_rot_speed->setValue ( 	config->readNumEntry("y_rot_speed", 2) );
	prefform->z_rot_speed->setValue ( 	config->readNumEntry("z_rot_speed", 1) );	
}

// save configuration
void Kcube::saveConfig()
{
	config->setGroup ("Kcube Applet");

	// Mouse button Click
	config->writeEntry ("lmb_click", 	prefform->lmb_click->text() );
	config->writeEntry ("rmb_click", 	prefform->rmb_click->text() );

	// 3D objects colors
	config->writeEntry ("obj_fg_color_l",	prefform->obj_fg_color_l->color() );
	config->writeEntry ("obj_fg_color_h", 	prefform->obj_fg_color_h->color() );	
	config->writeEntry ("obj_bg_color_l", 	prefform->obj_bg_color_l->color() );			
	config->writeEntry ("obj_bg_color_h", 	prefform->obj_bg_color_h->color() );
	
	// cpulabel colors
	config->writeEntry ("cpu_fg_color_l", 	prefform->cpu_fg_color_l->color() );
	config->writeEntry ("cpu_fg_color_h", 	prefform->cpu_fg_color_h->color() );	
	config->writeEntry ("cpu_bg_color_l", 	prefform->cpu_bg_color_l->color() );
	config->writeEntry ("cpu_bg_color_h", 	prefform->cpu_bg_color_h->color() );

	// high color cpu usage
	config->writeEntry ("high_color_cpu",	prefform->high_color_cpu->text() );
			
	// 3D Object
	config->writeEntry ("3d_object", 	prefform->s_3dobject->currentText() );

	// Refreshrate
	config->writeEntry ("r_rate",		prefform->r_rate->text() );

	// Rotations
	config->writeEntry ("s_x_rot",		prefform->s_x_rot->text() );
	config->writeEntry ("s_y_rot",		prefform->s_y_rot->text() );
	config->writeEntry ("s_z_rot",		prefform->s_z_rot->text() );
	config->writeEntry ("x_rot_speed",	prefform->x_rot_speed->value() );
	config->writeEntry ("y_rot_speed",	prefform->y_rot_speed->value() );
	config->writeEntry ("z_rot_speed",	prefform->z_rot_speed->value() );
		
	// save
	config->sync ();
}

void Kcube::mousePressEvent(QMouseEvent *me)
{
	if ( me->button() == QMouseEvent::LeftButton ) 
		KRun::runCommand( prefform->lmb_click->text() );
		
	if ( me->button() == QMouseEvent::RightButton )
		KRun::runCommand( prefform->rmb_click->text() );
}

void Kcube::setColors(QString load)
{
	if ( load == "low" ) {
		globject->setColors( 			prefform->obj_fg_color_l->color(), prefform->obj_bg_color_l->color() );
		label_load->setPaletteForegroundColor(	prefform->cpu_fg_color_l->color() );
		label_load->setPaletteBackgroundColor( 	prefform->cpu_bg_color_l->color() );
	}
	if ( load == "high" ) {
		globject->setColors( 			prefform->obj_fg_color_h->color(), prefform->obj_bg_color_h->color() );
		label_load->setPaletteForegroundColor( 	prefform->cpu_fg_color_h->color() );
		label_load->setPaletteBackgroundColor( 	prefform->cpu_bg_color_h->color() );
	}	
}
